@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000  Daemon Consulting Inc.
+ * Copyright (C) 2001  Daemon Consulting Inc.
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,6 +20,34 @@
 #define max(a,b) ((a) > (b) ? (a) : (b))
 #define min(a,b) ((a) < (b) ? (a) : (b))
 
+#ifndef MAP_FAILED
+#   define MAP_FAILED (-1)
+#endif
+
+#ifdef __GNUC__
+#define SPEEDY_INLINE __inline__
+#else
+#define SPEEDY_INLINE
+#endif
+
+#ifdef EWOULDBLOCK
+#   define SP_EWOULDBLOCK(e) ((e) == EWOULDBLOCK)
+#else
+#   define SP_EWOULDBLOCK(e) 0
+#endif
+#ifdef EAGAIN
+#   define SP_EAGAIN(e) ((e) == EAGAIN)
+#else
+#   define SP_EAGAIN(e) 0
+#endif
+#define SP_NOTREADY(e) (SP_EAGAIN(e) || SP_EWOULDBLOCK(e))
+
+typedef struct {
+    dev_t	d;
+    ino_t	i;
+} SpeedyDevIno;
+
+#include "speedy_util.h"
 #include "speedy_opt.h"
 #include "speedy_optdefs.h"
 #include "speedy_poll.h"
@@ -30,10 +58,5 @@
 #include "speedy_frontend.h"
 #include "speedy_file.h"
 #include "speedy_script.h"
-#include "speedy_util.h"
 #include "speedy_cb.h"
 #include "speedy_perl.h"
-
-#ifndef MAP_FAILED
-#   define MAP_FAILED (-1)
-#endif
