@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000  Daemon Consulting Inc.
+ * Copyright (C) 2001  Daemon Consulting Inc.
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,4 +21,8 @@ void speedy_script_find(slotnum_t *gslotnum_p, slotnum_t *sslotnum_p);
 int speedy_script_changed();
 void speedy_script_close();
 int speedy_script_open();
-struct stat *speedy_script_getstat();
+const struct stat *speedy_script_getstat();
+int speedy_script_open_failure();
+void speedy_script_munmap();
+SpeedyMapInfo *speedy_script_mmap(int max_size);
+void speedy_script_missing();
